#include <QOpenGLWindow>
#include <QMatrix4x4>

QT_BEGIN_NAMESPACE
class QOpenGLFramebufferObject;
class QOpenGLShaderProgram;
class QOpenGLBuffer;
class QOpenGLVertexArrayObject;
QT_END_NAMESPACE

class Window : public QOpenGLWindow
{
public:
    Window();
    ~Window();

    void initializeGL() Q_DECL_OVERRIDE;
    void resizeGL(int w, int h) Q_DECL_OVERRIDE;
    void paintGL() Q_DECL_OVERRIDE;

public slots:
    void readbackAndSave();

private:
    void initializeOffscreen();
    void initializeOnscreen();
    void setupOffscreenVertexAttribs();
    void paintOffscreen();
    void setupOnscreenVertexAttribs();
    void paintOnscreen();

    struct OffscreenData {
        OffscreenData() : rotationAngle(0), fbo(Q_NULLPTR), prog(Q_NULLPTR), vbo(Q_NULLPTR), vao(Q_NULLPTR) { }
        QMatrix4x4 projection;
        float rotationAngle;
        QOpenGLFramebufferObject *fbo;
        QOpenGLShaderProgram *prog;
        QOpenGLBuffer *vbo;
        QOpenGLVertexArrayObject *vao;
        int modelviewLoc;
        int projectionLoc;
    } offscreen;

    struct OnscreenData {
        OnscreenData() : rotationAngle(0), prog(Q_NULLPTR), vbo(Q_NULLPTR), vao(Q_NULLPTR) { }
        QMatrix4x4 projection;
        float rotationAngle;
        QOpenGLShaderProgram *prog;
        QOpenGLBuffer *vbo;
        QOpenGLVertexArrayObject *vao;
        int modelviewLoc;
        int projectionLoc;
    } onscreen;
};
